//
//  FileProviderItem.m
//  DemoSomeFiles
//
//  Created by Amy Worrall on 04/09/2017.
//  Copyright © 2017 Amy Worrall. All rights reserved.
//

#import "FileProviderItem.h"
#import <MobileCoreServices/MobileCoreServices.h>

@implementation FileProviderItem
{
  NSString *_id;
  NSString *_name;
  BOOL _isFolder;
  NSString *_parentFolderID;
  NSString *_owner;
  NSInteger _sizeInBytes;
  NSString *_mimeType;
  NSString *_downloadURL;
}

- (instancetype)initFolderWithFolderID:(NSString *)fbid name:(NSString *)name;
{
  if (!fbid || !name) {
    return nil;
  }
  if (self = [super init]) {
    _id = fbid;
    _name = name;
    _isFolder = YES;
  }
  return self;
}

- (instancetype)initFileWithFileID:(NSString *)fileID parentFolderID:(NSString *)folderID name:(NSString *)name owner:(NSString *)owner size:(NSInteger)bytes type:(NSString *)mimeType downloadURL:(NSString *)downloadURL;
{
  if (self = [super init]) {
    _id = fileID;
    _name = name;
    _parentFolderID = folderID;
    _owner = owner;
    _sizeInBytes = bytes;
    _mimeType = mimeType;
    _downloadURL = downloadURL;
  }
  return self;
}

- (NSFileProviderItemIdentifier)itemIdentifier {
  return _id;
}

- (NSFileProviderItemIdentifier)parentItemIdentifier {
  if (_isFolder) {
    return NSFileProviderRootContainerItemIdentifier;
  }
  return _parentFolderID;
}

- (NSFileProviderItemCapabilities)capabilities {
  return NSFileProviderItemCapabilitiesAllowsReading;
}

- (NSString *)filename {
  return _name;
}

- (NSString *)typeIdentifier {
  if (_isFolder) {
    return @"public.folder";
  }

  CFStringRef MIMEType = (__bridge CFStringRef)_mimeType;
  CFStringRef UTI = UTTypeCreatePreferredIdentifierForTag(kUTTagClassMIMEType, MIMEType, NULL);
  NSString *UTIString = (__bridge_transfer NSString *)UTI;
  return UTIString;
}



+ (instancetype)fromDictionary:(NSDictionary *)dict;
{
  FileProviderItem *item;

  if ([dict[@"isFolder"] boolValue]) {
    item = [[self alloc] initFolderWithFolderID:dict[@"id"]
                                           name:dict[@"name"]];
  } else {
    item = [[self alloc] initFileWithFileID:dict[@"id"]
                             parentFolderID:dict[@"parentFBID"]
                                       name:dict[@"name"]
                                      owner:dict[@"owner"]
                                       size:[dict[@"size"] integerValue]
                                       type:dict[@"type"]
                                downloadURL:dict[@"downloadURL"]];
  }
  return item;
}

- (NSDictionary *)toDictionary;
{
  NSMutableDictionary *dict = [NSMutableDictionary dictionary];

  dict[@"id"] = _id;
  dict[@"name"] = _name;

  if (_isFolder) {
    dict[@"isFolder"] = @YES;
  } else {
    dict[@"parentFBID"] = _parentFolderID;
    dict[@"owner"] = _owner;
    dict[@"size"] = @(_sizeInBytes);
    dict[@"type"] = _mimeType;
    dict[@"downloadURL"] = _downloadURL;
  }

  return dict;
}


@end
