//
//  WFGroupFilesEnumerator.m
//  WorkplaceFilesProvider
//
//  Created by Amy Worrall on 28/06/2017.
//  Copyright © 2017 Amy Worrall. All rights reserved.
//

#import "WFFolderContentsEnumerator.h"
#import "FilesGrabber.h"
#import "FileProviderItem.h"

@implementation WFFolderContentsEnumerator

- (instancetype)initWithEnumeratedItemIdentifier:(NSFileProviderItemIdentifier)enumeratedItemIdentifier {
  if (self = [super init]) {
    _enumeratedItemIdentifier = enumeratedItemIdentifier;
  }
  return self;
}

- (void)invalidate {
}

- (void)enumerateItemsForObserver:(id<NSFileProviderEnumerationObserver>)observer startingAtPage:(NSFileProviderPage)page {

  FilesGrabber *grabber = [[FilesGrabber alloc] init];
  grabber.folderID = self.enumeratedItemIdentifier;

  __weak typeof(grabber) weakGrabber = grabber;
  grabber.completionBlock = ^{
    NSMutableArray *fileProviderItems = [NSMutableArray new];
    for (NSDictionary *fileNode in weakGrabber.files) {
      FileProviderItem *fileProviderItem =
      [[FileProviderItem alloc] initFileWithFileID:fileNode[@"id"]
                                      parentFolderID:self.enumeratedItemIdentifier
                                                name:fileNode[@"name"]
                                               owner:fileNode[@"owner"]
                                                size:[fileNode[@"size"] integerValue]
                                                type:fileNode[@"file_type"]
                                         downloadURL:fileNode[@"download_url"]];
      [fileProviderItems addObject:fileProviderItem];

      [self.delegate setEntry:fileProviderItem forFBID:fileNode[@"id"]];
    }

    [observer didEnumerateItems:fileProviderItems];
    [observer finishEnumeratingUpToPage:nil];
  };
  [[NSOperationQueue mainQueue] addOperation:grabber];

}

- (void)enumerateChangesForObserver:(id<NSFileProviderChangeObserver>)observer fromSyncAnchor:(NSFileProviderSyncAnchor)anchor {
  /* TODO:
   - query the server for updates since the passed-in sync anchor

   If this is an enumerator for the active set:
   - note the changes in your local database

   - inform the observer about item deletions and updates (modifications + insertions)
   - inform the observer when you have finished enumerating up to a subsequent sync anchor
   */
}

@end
